"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.setOriginalRule = void 0;

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _reactDom = _interopRequireDefault(require("react-dom"));

var _moment = _interopRequireDefault(require("moment"));

var _domScrollIntoView = _interopRequireDefault(require("dom-scroll-into-view"));

var _utils = _interopRequireDefault(require("../utils/utils"));

function computedStyle(el, prop) {
  var getComputedStyle = window.getComputedStyle;
  var style = // If we have getComputedStyle
  getComputedStyle ? // Query it
  // TODO: From CSS-Query notes, we might need (node, null) for FF
  getComputedStyle(el) : // Otherwise, we are in IE and use currentStyle
  el.currentStyle;

  if (style) {
    // Switch to camelCase for CSSOM
    // DEV: Grabbed from jQuery
    // https://github.com/jquery/jquery/blob/1.9-stable/src/css.js#L191-L194
    // https://github.com/jquery/jquery/blob/1.9-stable/src/core.js#L593-L597
    return style[prop.replace(/-(\w)/gi, function (word, letter) {
      return letter.toUpperCase();
    })];
  }

  return undefined;
}

function getScrollableContainer(n) {
  var node = n;
  var nodeName;
  /* eslint no-cond-assign:0 */

  while ((nodeName = node.nodeName.toLowerCase()) !== 'body') {
    var overflowY = computedStyle(node, 'overflowY'); // https://stackoverflow.com/a/36900407/3040605

    if (node !== n && (overflowY === 'auto' || overflowY === 'scroll') && node.scrollHeight > node.clientHeight) {
      return node;
    }

    node = node.parentNode;
  }

  return nodeName === 'body' ? node.ownerDocument : node;
}

var originRules = {
  DateTimeInput: function DateTimeInput() {
    var field = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    var rv;
    var withDefault = false;
    var _field$property = field.property,
        property = _field$property === void 0 ? {} : _field$property;
    var valueFormat = property.valueFormat,
        valueDefault = property.valueDefault;
    var now = new Date();

    if (valueDefault === 'NOW') {
      rv = {
        currentTime: undefined,
        timeRange: undefined
      };
      var defaultValue;
      withDefault = true;

      switch (property.selectionModule) {
        case 'SINGLE':
          defaultValue = valueFormat === 'YYYY-MM' ? new Date(now.getFullYear(), now.getMonth(), 1) : now;
          rv.currentTime = defaultValue.valueOf();
          break;

        case 'PAIR':
          defaultValue = now;
          var startDate = (0, _moment.default)(now).startOf('day');
          var endDate = (0, _moment.default)(now).endOf('day');
          rv.timeRange = {
            startDate: startDate.valueOf(),
            endDate: endDate.valueOf()
          };
          break;

        default:
          break;
      }
    }

    return {
      hasDefault: withDefault,
      value: rv
    };
  }
};

var setOriginalRule = function setOriginalRule(name, fn) {
  if (name && typeof fn === 'function' && !originRules[name]) {
    originRules[name] = fn;
  }
};

exports.setOriginalRule = setOriginalRule;

var Field = function Field() {
  var _this = this;

  var props = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  (0, _classCallCheck2.default)(this, Field);

  this.setDefaultHandle = function (fn) {
    if (typeof fn === 'function') _this.getDefaultFn = fn;
  };

  this.setWatch = function (fn, id) {
    if (typeof fn === 'function') _this.watch[id ? id : _utils.default.makeId()] = fn;
  };

  this.getDefault = function () {
    return _this.getDefaultFn ? _this.getDefaultFn(_this.config) : undefined;
  };

  this.setValue = function (val) {
    _this.value = val;
  };

  this.setValidator = function (v) {
    _this.validator = v;
  };

  this.scroll = function () {
    if (_this.fieldWrapper) {
      var node = _reactDom.default.findDOMNode(_this.fieldWrapper);

      if (node) {
        var parentNode = getScrollableContainer(node);
        (0, _domScrollIntoView.default)(node, parentNode, {
          onlyScrollIfNeeded: true
        });
      }
    }
  };

  this.validate = function () {
    var fieldName = _this.name;
    return new Promise(function (resolve, reject) {
      _this.validator.validate((0, _defineProperty2.default)({}, fieldName, _this.value), {
        formFields: [_this.config]
      }, function (errors, fields) {
        if (errors) {
          _this.validateError = fields[fieldName] && fields[fieldName].message;
        } else {
          _this.validateError = undefined;
        }

        resolve({
          errors: errors,
          fields: fields
        });
      });
    });
  };

  this.updateConfig = function (config) {
    _this.config = config;
  };

  var _config = props.config,
      defaultHandle = props.defaultHandle;
  this.config = _config;
  this.watch = {};
  this.value = undefined;
  this.name = _config.identifier;
  this.validateError = undefined;
  this.componentInstance = undefined;
  this.fieldWrapper = undefined; // 默认值函数        

  if (typeof defaultHandle === 'function') {
    this.getDefaultFn = defaultHandle;
  } else {
    if (originRules[_config.type]) this.getDefaultFn = originRules[_config.type];
  }
};

var _default = Field;
exports.default = _default;