"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _lodash = _interopRequireDefault(require("lodash"));

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) { symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); } keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { (0, _defineProperty2.default)(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

var linkedFieldTypes = ['AssociationInput', 'ExternalOptionInput', 'CollectionAccountInput', 'AccountInput', 'DatahubExpenseRuleInput'];
/**
 * @param {Object} param
 * @param {function} param.formDataPipe 数据互联异步请求的主题参数的管道函数（过滤，转化等动作） 
 * @param {function} param.parentFormDataPipe 费用种数据互联需要带上报销单C级字段
*/

var genRuleAssociateValue = function genRuleAssociateValue(_ref) {
  var formFields = _ref.formFields,
      receiptTypes = _ref.receiptTypes,
      detailIdentifier = _ref.detailIdentifier,
      fetchAssociateData = _ref.fetchAssociateData,
      fetchExternalOptionData = _ref.fetchExternalOptionData,
      fetchDatahubExpenseRule = _ref.fetchDatahubExpenseRule,
      formDataPipe = _ref.formDataPipe,
      parentFormDataPipe = _ref.parentFormDataPipe,
      _ref$ignoreDependence = _ref.ignoreDependence,
      ignoreDependence = _ref$ignoreDependence === void 0 ? false : _ref$ignoreDependence,
      conTextLegalEntity = _ref.conTextLegalEntity,
      selectionIndex = _ref.selectionIndex;
  var formulaRules = [];
  var defaultReceiptType = receiptTypes.default;

  _lodash.default.forEach(formFields, function (field) {
    if (linkedFieldTypes.includes(field.type)) {
      if (['CollectionAccountInput', 'AccountInput'].includes(field.type) && field.property.accountFrom === 'INTERNAL') return;
      /**
           * 数据互联获取数据的异步请求
           * @param {Object} formData formchef的最新表单数据
           * @param {Object} fieldData referField变量的信息，格式为{code,name}
           * @param {Object} extraParams
           * @param {number} extraParams.formIndex 表单的顺序索引
           * @param {Object[]} extraParams.formFields 最新的表单配置信息                 
           */

      var asyncFn = function asyncFn(formData, fieldData, _ref2) {
        var formIndex = _ref2.formIndex,
            latestFormFields = _ref2.formFields;
        var _field$property = field.property,
            dataLinkApi = _field$property.dataLinkApi,
            dataSource = _field$property.dataSource,
            appBusinessScope = _field$property.appBusinessScope,
            dependsOnReceipt = _field$property.dependsOnReceipt;
        var receiptTypeOfParam = receiptTypes[field.type] ? receiptTypes[field.type] : defaultReceiptType;
        var params = {
          fetchFormData: function fetchFormData() {
            return typeof formDataPipe === 'function' ? formDataPipe(latestFormFields, formData, field, formIndex) : formData;
          },
          fetchParentFormData: typeof parentFormDataPipe === 'function' ? function () {
            return parentFormDataPipe();
          } : undefined,
          // NOTE 不写成上一行那种形式
          receiptType: receiptTypeOfParam,
          detailIdentifier: detailIdentifier && detailIdentifier.startsWith('MOCK_') ? undefined : detailIdentifier,
          selectionIndex: selectionIndex !== null && selectionIndex !== void 0 ? selectionIndex : formIndex,
          legalEntity: conTextLegalEntity,
          //支付场景中外部档案会被公司抬头影响
          dependsOnReceipt: dependsOnReceipt
        };
        var api;

        if (field.type === 'ExternalOptionInput') {
          //外部档案                        
          api = fetchExternalOptionData;
          params.externalReferenceCode = dataSource;
        } else if (field.type === 'DatahubExpenseRuleInput') {
          // 用车制度
          api = fetchDatahubExpenseRule;
          params.appBusinessScope = appBusinessScope;
        } else {
          //数据互联
          api = fetchAssociateData;
          params.targetUrl = dataLinkApi;
        }

        return api({
          params: params
        }).then(function (resp) {
          if (!resp) return;
          var definedList = resp.definedList,
              defaultValue = resp.defaultValue;
          var result = {}; // defaultValue无值属于正常现象

          if (defaultValue) {
            var data = {
              definedList: definedList,
              value: defaultValue && [defaultValue] || []
            };

            if (['CollectionAccountInput', 'AccountInput'].includes(field.type)) {
              data.accountFrom = 'DATA_ASSOSIATION';
            }

            if (['ExternalOptionInput', 'DatahubExpenseRuleInput'].includes(field.type)) {
              var primary = _lodash.default.find(definedList, {
                primary: true
              });

              var extraValue = {
                bizCode: defaultValue && defaultValue['bizCode'],
                name: defaultValue && defaultValue['name' || primary.businessCode]
              };
              data = _objectSpread(_objectSpread(_objectSpread({}, data), defaultValue || {}), extraValue);

              if (field.type === 'ExternalOptionInput') {
                data.referenceDataCode = dataSource;
              }
            }

            result = {
              status: 'success',
              data: data
            };
          } else {
            result = {
              status: 'success',
              data: undefined
            };
          }

          return result;
        });
      };

      _lodash.default.forEach(field.cascadeDataRelations, function (referField) {
        var rule = {
          targetField: field.identifier,
          referField: referField.value,
          asyncFn: asyncFn
        };
        formulaRules.push(rule);
      });

      if (ignoreDependence) {
        formulaRules.push({
          targetField: field.identifier,
          referField: '$associateDefault$',
          asyncFn: asyncFn
        });
      }
    }
  });

  return formulaRules;
};

var _default = genRuleAssociateValue;
exports.default = _default;