### OptionsModal

> 基础选择组件
>
> 代码维护者：孔佳承

##### 功能说明

1. 可用于多项数据展示和选择，支持单选、多选和默认选中。

##### 代码演示

1. 在页面中 `modalManage` 中使用

   ```jsx
   import modalManage from '@u/modalManage';
   import { OptionsModal } from 'maycur-business';
   
   modalManage.open({
       component: OptionsModal,
       componentProps: {
       		dataValueKey: 'code', // 选项的真实值；默认值：'code'
         	dataLabelKey: 'label', // 选项展示的值；默认值：'label'
         	options：options, // 选项列表
         	title: '标题', // Modal 的标题；默认值：'标题'
         	isRadio: true, // 单选 or 多选；默认值：true
         	wrapWidth: 520, // Modal 的宽度；默认值为：520
         	contentMaxHeight: 250, // 选项区域的 max-height；默认值：250
         	defaultSelectedKeys: [], // 自己传入的默认选中值，保留自主性
         	alertObj: (
                       <Alert
                           banner
                           message="提示文字"
                           showIcon
                           type="info"
                       />
                   ), // 自己传入的 Alert 对象，保留自主性
       },
   }).then(_options => {
     	// _options：从传入的 options 中选项的一项或多项
   })
   ```
   
2. API

   | 属性                | 说明                             | 类型            | 默认值  |
   | ------------------- | -------------------------------- | --------------- | ------- |
   | dataValueKey        | 选项的真实值                     | String          | 'code'  |
   | dataLabelKey        | 选项展示的值                     | String          | 'label' |
   | options             | 选项数据列表                     | Array \| Object | -       |
   | title               | Modal 的标题                     | String          | '标题'  |
   | isRadio             | 是否为单选                       | Boolean         | true    |
   | wrapWidth           | Modal 的宽度                     | Number          | 520     |
   | contentMaxHeight    | 选项区域的 max-height            | Number          | 250     |
   | defaultSelectedKeys | 默认选中的选项的 dataValueKey 值 | Array           | -       |
   | alertObj            | 自己传入的 Alert 对象            | Object          | -       |

3. 注意事项

   - 单选的情况下，返回值 `_options`为`Object` 类型；
     多选的情况下，返回值 `_options`为`Array`类型。
   - 单选的情况下，若传入`defaultSelectedKeys`，选中`defaultSelectedKeys[0]`；
     多选的情况下，若传入`defaultSelectedKeys`，则尽数选中。
   - `dataValueKey`和`dataLabelKey`必须和`options`数据对象中的属性名对应。


