"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

var _typeof2 = _interopRequireDefault(require("@babel/runtime/helpers/typeof"));

var hasOwnProperty = Object.prototype.hasOwnProperty;

function isObject(value) {
  var type = (0, _typeof2.default)(value);
  return value != null && (type === 'object' || type === 'function');
}

function isArray(value) {
  if (!checkEmpty(value)) return false;
  return Array.isArray(value);
}
/* 只对Array和Object有效 */


function isEmpty(value) {
  if (!checkEmpty(value)) return true;

  if (isArray(value)) {
    return value.length === 0;
  } else if (isObject(value)) {
    return Object.keys(value).length === 0;
  }
}

function checkEmpty(par) {
  if (par === undefined) return false;
  if (par === null) return false;
  if (!par) return false;
  return true;
}

function has(object, key) {
  return object != null && hasOwnProperty.call(object, key);
}

function forEach(collection, iteratee) {
  if (collection === undefined || collection === null) return;

  if (Array.isArray(collection)) {
    return collection.forEach(iteratee);
  } else if (isObject(collection)) {
    Object.keys(collection).forEach(function (key) {
      if (typeof iteratee === 'function') {
        iteratee(collection[key], key);
      }
    });
  }
}

function union() {
  var array1 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
  var array2 = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
  if (!array1) return array2;
  if (!array2) return array1;
  return (0, _toConsumableArray2.default)(new Set([].concat((0, _toConsumableArray2.default)(array1), (0, _toConsumableArray2.default)(array2))));
}

function find(par, iteratee) {
  if (checkEmpty(par) && Array.isArray(par)) {
    return par.find(iteratee);
  }
}

function findIndex(par, iteratee) {
  if (checkEmpty(par) && Array.isArray(par)) {
    return par.findIndex(iteratee);
  }
}

function last(value) {
  if (!isArray(value)) return;
  if (isEmpty(value)) return;
  return value[value.length - 1];
}

function includes(collection, name) {
  if (!isArray(collection) || !checkEmpty(name)) return false;

  if (isArray(collection)) {
    return collection.includes(name);
  } else {
    return false;
  }
}

function map(array, iteratee) {
  var index = -1;
  var length = array == null ? 0 : array.length;
  var result = new Array(length);

  while (++index < length) {
    result[index] = iteratee(array[index], index, array);
  }

  return result;
}

function pick(object, keys) {
  return keys.reduce(function (obj, key) {
    if (object && object.hasOwnProperty(key)) {
      obj[key] = object[key];
    }

    return obj;
  }, {});
}

function debounce(func, wait, immediate) {
  var timeout;
  return function () {
    var context = this,
        args = arguments;
    clearTimeout(timeout);
    timeout = setTimeout(function () {
      timeout = null;
      if (!immediate) func.apply(context, args);
    }, wait);
    if (immediate && !timeout) func.apply(context, args);
  };
}

function difference(targetArray, matchedArray) {
  var arrays = [targetArray, matchedArray];
  return arrays.reduce(function (a, b) {
    return a.filter(function (value) {
      return b.includes(value);
    });
  });
}

var slimLodash = {
  has: has,
  forEach: forEach,
  union: union,
  find: find,
  isObject: isObject,
  isArray: isArray,
  isEmpty: isEmpty,
  last: last,
  includes: includes,
  checkEmpty: checkEmpty,
  map: map,
  findIndex: findIndex,
  pick: pick,
  debounce: debounce,
  difference: difference
};
var _default = slimLodash;
exports.default = _default;