"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.setEndNotification = exports.setForegroundNotification = exports.checkAndroidBackgroundMode = exports.showToast = exports.recordPointSuccess = exports.pauseAmapTrack = exports.stopAmapTrack = exports.startAmapTrack = exports.initStartAmapTrack = exports.getCurrentPoisition = exports.manageInitPoints = exports.timeFormate = exports.milePrefix = exports.POINT_END = exports.POINT_VIA = exports.POINT_START = void 0;

require("maycur-antd/lib/modal/style/css");

var _modal = _interopRequireDefault(require("maycur-antd/lib/modal"));

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

var _lodash = require("lodash");

var _utils = _interopRequireDefault(require("../utils/utils"));

var POINT_START = "START";
exports.POINT_START = POINT_START;
var POINT_VIA = "VIA";
exports.POINT_VIA = POINT_VIA;
var POINT_END = "END";
exports.POINT_END = POINT_END;
var milePrefix = _utils.default.prefixCls + "-mileage";
exports.milePrefix = milePrefix;

var timeFormate = function timeFormate(time) {
  var date = new Date(time);
  return date.getFullYear() + "-" + ("0" + (date.getMonth() + 1)).slice(-2) + "-" + ("0" + date.getDate()).slice(-2) + " " + ("0" + date.getHours()).slice(-2) + ":" + ("0" + date.getMinutes()).slice(-2);
};

exports.timeFormate = timeFormate;

var manageInitPoints = function manageInitPoints(init) {
  if (!init || init.length === 0) {
    // 里程未开始，无start
    return [{
      code: new Date().getTime() + "s",
      type: POINT_START
    }, {
      code: new Date().getTime() + "e",
      type: POINT_END
    }];
  } else {
    var end = (0, _lodash.find)(init, {
      type: POINT_END
    }); // 里程未结束，无end

    if (!end || !end.address) {
      return [].concat((0, _toConsumableArray2.default)(init), [{
        code: new Date().getTime() + "",
        type: POINT_VIA
      }, {
        code: new Date().getTime() + "e",
        type: POINT_END
      }]);
    }

    return init;
  }
};
/**
 * JSBridge 获取定位信息
 */


exports.manageInitPoints = manageInitPoints;

var getCurrentPoisition = function getCurrentPoisition() {
  return new Promise(function (resolve, reject) {
    if (window.cordova) {
      window.cordova.plugins.AmapTrack.getCurrentPosition(null, function (res) {
        res.timestamp = new Date().getTime();
        res.addrDescp = res.address;
        res.address = res.poiname;
        res.lat = res.latitude;
        res.long = res.longitude;
        console.log(res);
        resolve(res);
      }, function () {
        reject();
      });
    } else {
      // 待修改
      var point = {
        timestamp: new Date().getTime(),
        address: "mock address",
        addrDescp: "mock addrDescp",
        lat: 30.25924446,
        long: 120.21937542
      };
      resolve(point);
    }
  });
};
/**
 * JSBridge 初始化并开启实时路径打点
 */


exports.getCurrentPoisition = getCurrentPoisition;

var initStartAmapTrack = function initStartAmapTrack(info) {
  return new Promise(function (resolve, reject) {
    checkAndroidBackgroundMode().then(function () {
      if (window.cordova) {
        initAmapTrack(info).then(function () {
          startAmapTrack().then(function () {
            resolve();
          });
        }, function () {
          reject();
        });
      } else {
        resolve();
      }
    }, function () {
      reject();
    });
  });
};

exports.initStartAmapTrack = initStartAmapTrack;

var initAmapTrack = function initAmapTrack(info) {
  return new Promise(function (resolve, reject) {
    if (window.cordova && info.serviceId && info.terminalId && info.traceId) {
      window.cordova.plugins.AmapTrack.init(info, function (res) {
        resolve(res);
      }, function () {
        reject();
      });
    }
  });
};
/**
 * JSBridge 继续实时路径打点
 */


var startAmapTrack = function startAmapTrack() {
  return new Promise(function (resolve, reject) {
    if (window.cordova) {
      window.cordova.plugins.AmapTrack.startTrack(null, function (res) {
        resolve(res);
      }, function () {
        reject();
      });
    } else {
      resolve();
    }
  });
};
/**
 * JSBridge 结束实时路径打点
 */


exports.startAmapTrack = startAmapTrack;

var stopAmapTrack = function stopAmapTrack() {
  if (window.device && window.device.platform === "Android") {
    //定位结束可以关闭后台模式
    window.cordova.plugins.backgroundMode.disable(); //设置结束后的通知栏

    setEndNotification();
  }

  return new Promise(function (resolve, reject) {
    if (window.cordova) {
      window.cordova.plugins.AmapTrack.stopService(null, function (res) {
        resolve(res);
      }, function () {
        reject();
      });
    } else {
      resolve();
    }
  });
};
/**
 * JSBridge 暂停实时路径打点
 */


exports.stopAmapTrack = stopAmapTrack;

var pauseAmapTrack = function pauseAmapTrack() {
  return new Promise(function (resolve, reject) {
    if (window.cordova) {
      window.cordova.plugins.AmapTrack.stopTrack(null, function (res) {
        resolve(res);
      }, function () {
        reject();
      });
    } else {
      resolve();
    }
  });
}; // 添加/删除途经点占位符（“记录途经点”按钮）


exports.pauseAmapTrack = pauseAmapTrack;

var recordPointSuccess = function recordPointSuccess(pt, newPoints) {
  if (pt.type !== POINT_END) {
    newPoints.splice(newPoints.length - 1, 0, {
      code: new Date().getTime(),
      type: POINT_VIA
    });
  } else {
    var noiseIdx = (0, _lodash.findIndex)(newPoints, function (p) {
      return p.type === POINT_VIA && !p.address;
    });
    if (noiseIdx !== -1) newPoints.splice(noiseIdx, 1);
  }

  return newPoints;
};

exports.recordPointSuccess = recordPointSuccess;

var showToast = function showToast(text) {
  var modal = _modal.default.success({
    // title: 'This is a notification message',
    title: text
  }); // setTimeout(() => {
  //   modal.destroy();
  // }, 1000);

};

exports.showToast = showToast;

var checkAndroidBackgroundMode = function checkAndroidBackgroundMode() {
  return new Promise(function (resolve, reject) {
    if (!window.cordova || !window.cordova.plugins.backgroundMode) {
      resolve();
    } else {
      var backgroundMode = window.cordova.plugins.backgroundMode;

      if (window.device.platform === "Android") {
        //设置安卓后台运行模式
        backgroundMode.enable(); //设置监听

        backgroundMode.on("activate", function () {
          backgroundMode.disableWebViewOptimizations();
        }); //判断通知是否开启

        backgroundMode.isOpenNotification(function (status) {
          //true or false
          if (!status) {
            alert("通知未起来，将跳转到通知设置页开启"); //如果未开启，则先弹框告知用户，让用户在设置中将通知打开，然后调用下面的方法自动跳转到设置页

            backgroundMode.openNotificationSettings();
            reject();
          } else {
            //设置安卓前台状态栏默认通知样式
            setForegroundNotification(); //判断是否忽略了电池优化

            backgroundMode.isIgnoringBatteryOptimizations(function (isIgnoring) {
              if (!isIgnoring) {
                alert("电池优化未关闭，请稍后点击允许关闭电池优化"); //如果没有忽略，代表使用了系统推荐的电池优化，则应用在后台时会容易被杀掉或者gps停掉，这时候用下面这个方法跳转到应用电池优化界面

                backgroundMode.openBatteryOptimizationsSettings();
                reject();
              } else {
                resolve();
              }
            });
          }
        });
      } else {
        resolve();
      }
    }
  });
};

exports.checkAndroidBackgroundMode = checkAndroidBackgroundMode;

var setForegroundNotification = function setForegroundNotification() {
  var startConfig = {
    title: "每刻报销",
    text: "每刻报销正在使用地理定位",
    subText: "",
    // see https://developer.android.com/reference/android/support/v4/app/NotificationCompat.Builder.html#setSubText(java.lang.CharSequence)
    icon: "icon",
    // this will look for icon.png in platforms/android/res/drawable|mipmap
    color: "1199f0",
    // hex format like 'F14F4D'
    resume: true,
    hidden: false,
    bigText: false,
    channelName: "每刻报销推送渠道",
    // Shown when the user views the app's notification settings
    channelDescription: "",
    // Shown when the user views the channel's settings
    allowClose: false,
    // add a "Close" action to the notification
    closeIcon: "power",
    // An icon shown for the close action
    closeTitle: "Close",
    // The text for the close action
    showWhen: true,
    //(Default: true) Show the time since the notification was created
    visibility: "private" // Android only: one of 'private' (default), 'public' or 'secret' (see https://developer.android.com/reference/android/app/Notification.Builder.html#setVisibility(int))

  };
  window.cordova.plugins.backgroundMode.setDefaults(startConfig);
};

exports.setForegroundNotification = setForegroundNotification;

var setEndNotification = function setEndNotification() {
  var endConfig = {
    title: "每刻报销",
    text: "每刻报销已停止持续使用地理定位",
    subText: "",
    // see https://developer.android.com/reference/android/support/v4/app/NotificationCompat.Builder.html#setSubText(java.lang.CharSequence)
    icon: "icon",
    // this will look for icon.png in platforms/android/res/drawable|mipmap
    color: "1199f0",
    // hex format like 'F14F4D'
    resume: true,
    hidden: false,
    bigText: false,
    channelName: "每刻报销推送渠道",
    // Shown when the user views the app's notification settings
    channelDescription: "",
    // Shown when the user views the channel's settings
    allowClose: true,
    // add a "Close" action to the notification
    closeIcon: "power",
    // An icon shown for the close action
    closeTitle: "Close",
    // The text for the close action
    showWhen: false,
    //(Default: true) Show the time since the notification was created
    visibility: "private" // Android only: one of 'private' (default), 'public' or 'secret' (see https://developer.android.com/reference/android/app/Notification.Builder.html#setVisibility(int))

  };
  window.cordova.plugins.backgroundMode.configure(endConfig);
};

exports.setEndNotification = setEndNotification;