"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _lodash = _interopRequireDefault(require("lodash"));

function commonComparator(sourceList, fieldValue, isPositive) {
  var _fieldValue;

  if (_lodash.default.isArray(fieldValue)) {
    _fieldValue = fieldValue[0];
  } else if (_lodash.default.isObject(fieldValue)) {
    _fieldValue = fieldValue.code;
  } else {
    _fieldValue = fieldValue;
  }

  var bool = _lodash.default.indexOf(sourceList, _fieldValue) !== -1;
  return isPositive ? bool : !bool;
}
/**
 * 部门选择控件里面返回的fieldValue的数据结构如下,并且唯一标识是code
 * [{code}]
 */


function departmentComparator(sourceList, fieldValue, isPositive) {
  return _lodash.default.every(Array.isArray(fieldValue) ? fieldValue : [fieldValue], function (item) {
    var bool = _lodash.default.intersection(_lodash.default.split(sourceList, ','), _lodash.default.split(item.heirCode, '.')).length > 0;
    return isPositive ? bool : !bool;
  });
}
/**
 * 部门选择控件(不包含子部门)里面返回的fieldValue的数据结构如下,并且唯一标识是code
 * [{code}]
 */


function departmentComparatorWithOutChild(sourceList, fieldValue, isPositive) {
  return _lodash.default.every(Array.isArray(fieldValue) ? fieldValue : [fieldValue], function (item) {
    var bool = _lodash.default.intersection(_lodash.default.split(sourceList, ','), [item.code]).length > 0;
    return isPositive ? bool : !bool;
  });
}
/**
 * 找到两个数组的相同项
 */


function findMatchItems(array1, array2) {
  return _lodash.default.filter(array1, function (group) {
    return _lodash.default.indexOf(array2, group) !== -1;
  });
}
/**
 * 人员选择控件里面返回的fieldValue的数据结构如下,并且唯一标识是userCode
 * [{userCode}]
 */


function staffComparator(sourceString, fieldValue, isPositive, extraConfig) {
  var sourceObjects = sourceString ? JSON.parse(sourceString) : [];

  var userList = _lodash.default.map(_lodash.default.filter(sourceObjects, {
    type: 'STAFF'
  }), 'code');

  var userGroupList = _lodash.default.map(_lodash.default.filter(sourceObjects, {
    type: 'USER_GROUP'
  }), 'code');

  var _fieldValue = Array.isArray(fieldValue) ? fieldValue : [fieldValue];

  if (!userGroupList.length) {
    return _lodash.default.every(_fieldValue, function (item) {
      var bool = _lodash.default.indexOf(userList, item && item.userCode) !== -1;
      return isPositive ? bool : !bool;
    });
  }

  return extraConfig.fetchUserUserGroup(_lodash.default.map(_fieldValue, 'userCode')).then(function (resp) {
    var userGroupMap = resp.data;
    return _lodash.default.every(_fieldValue, function (item) {
      var bool = _lodash.default.indexOf(userList, item && item.userCode) !== -1;

      if (isPositive) {
        //如果是属于的情况，只有在人员已经在userList里面没有的情况下，才需要再判断userGroupList里面有没有
        if (!bool) {
          var match = findMatchItems(_lodash.default.map(userGroupMap[item.userCode], 'code'), userGroupList);
          bool = match.length > 0;
        }
      } else {
        //如果是不属于的情况，只有在人员在userList没有的情况下，才需要再判断userGroupList里面有没有
        if (!bool) {
          var _match = findMatchItems(_lodash.default.map(userGroupMap[item.userCode], 'code'), userGroupList);

          bool = _match.length !== 0;
        }
      }

      return isPositive ? bool : !bool;
    });
  });
}
/**
 * 数据互联返回的fieldValue数据结构如下，唯一标识为id
 * {value: [{id}]}
 */


function assositionComparator(sourceList, fieldValue, isPositive) {
  if (_lodash.default.isEmpty(fieldValue && fieldValue.value)) {
    return false;
  } else {
    return _lodash.default.every(fieldValue && fieldValue.value, function (item) {
      var bool = _lodash.default.indexOf(sourceList, item && item.id) !== -1;
      return isPositive ? bool : !bool;
    });
  }
}
/**
 * 表单控件里面返回的fieldValue的数据结构如下
 * {details: [{level}]}
 */


function validateComparator(sourceList, fieldValue, isPositive) {
  return _lodash.default.every(fieldValue && fieldValue.details, function (item) {
    //return item.level === sourceList[0];
    var bool = item.level === sourceList[0];
    return isPositive ? bool : !bool;
  });
}

function comparaRules(condition, data, isPositive, extraConfig) {
  var field = condition.field,
      identifier = condition.identifier;
  var fieldValue = data[identifier];

  var sourceList = _lodash.default.split(condition.valueCodes, ','); //当值为空的时候，属于条件返回false，不属于返回true


  if (_lodash.default.isEmpty(fieldValue)) {
    return !isPositive;
  }

  var result;

  switch (field) {
    case 'WEB_CHECK_COVER_DEPARTMENT_CODE_WITH_OUT_CHILD':
      // 不含下级部门
      result = departmentComparatorWithOutChild(sourceList, fieldValue, isPositive);
      break;

    case 'WEB_CHECK_COVER_DEPARTMENT_CODE':
      // 含下级部门
      result = departmentComparator(sourceList, fieldValue, isPositive);
      break;

    case 'WEB_CHECK_COVER_USER_CODE':
      result = staffComparator(condition.value, fieldValue, isPositive, extraConfig);
      break;

    case 'WEB_CHECK_STAFF_CODE':
      result = staffComparator(condition.value, fieldValue, isPositive, extraConfig);
      break;

    case 'WEB_CHECK_DATA_ASSOCIATION':
      result = assositionComparator(sourceList, fieldValue, isPositive);
      break;

    case 'WEB_CHECK_FORM_VALIDATION':
      result = validateComparator(sourceList, fieldValue, isPositive);
      break;

    default:
      result = commonComparator(sourceList, fieldValue, isPositive);
      break;
  }

  return result;
}
/**
 * 
 * @param {*} condition 条件
 * @param {*} data 表单数据对象
 * @param {*} isPositive 是否包含
 */


function IN(condition, data, isPositive, extraConfig) {
  return comparaRules(condition, data, isPositive, extraConfig);
}

var _default = IN;
exports.default = _default;