"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _slimLodash = _interopRequireDefault(require("../utils/slimLodash"));

var _genAssociatedFormula = _interopRequireDefault(require("./genAssociatedFormula"));

var _genShareFieldFormula = _interopRequireDefault(require("./genShareFieldFormula"));

var _formulaParser = _interopRequireDefault(require("../utils/formulaParser"));

var _genFormulaFieldFormula = _interopRequireDefault(require("./genFormulaFieldFormula"));

/*
 * @Description: file content
 * @Author: RongWei
 * @Date: 2019-08-06 10:55:01
 * @LastEditors: Please set LastEditors
 * @LastEditTime: 2020-10-09 17:30:17
 */

/**variableInfo说明
 * name： 名称，必须
 * defaultValue： 默认值 
 * handle：转化函数
 */

/** 公式模型
* targetField:公式对应的结果对象，为一个表单对象
* referField:公式相关的表单变量（区别于自定义变量）,只允许单个，多个增加逻辑复杂度
* formula:公式内容
* privateParser:是否使用私有编译器
* registerVariables:初始化的自定义变量
* setFunctionList:自定义的函数
* updatePool:对私有编译器的数据池(pool),进项数据过滤（只在privateParser=true有效）
*/

/* test block */

/* test block end*/
var FormChefParser = function FormChefParser() {
  var _this = this;

  (0, _classCallCheck2.default)(this, FormChefParser);

  this.registerVariable = function (parserName) {
    var variableInfo = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    var keepPre = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;
    if (parserName && !_this.childParsers[parserName]) return;
    var parserCollection = parserName ? _this.childParsers[parserName] : _this.mainParser;
    if (!parserCollection.parser || !variableInfo) return;

    if (!(parserCollection.parser instanceof _formulaParser.default)) {
      return;
    }

    var name = variableInfo.name,
        defaultValue = variableInfo.defaultValue,
        handle = variableInfo.handle,
        parent = variableInfo.parent;

    if (name) {
      if (_slimLodash.default.has(parserCollection.variables, name)) {
        if (keepPre) parserCollection.parser.setVariable(name, defaultValue);
      } else {
        parserCollection.variables[name] = {
          name: name,
          defaultValue: defaultValue,
          handle: handle,
          parent: parent
        };
        parserCollection.parser.setVariable(name, defaultValue);
      }
    }
  };

  this.registerParser = function (name) {
    if (!_slimLodash.default.has(_this.childParsers, name)) {
      _this.childParsers[name] = {
        name: name,
        parser: new _formulaParser.default(),
        variables: {},
        functions: {}
      };
    }
  };

  this.registerFunction = function (parserName) {
    var functionInfo = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    if (parserName && !_this.childParsers[parserName]) return;
    var parserCollection = parserName ? _this.childParsers[parserName] : _this.mainParser;
    var parser = parserCollection.parser,
        functions = parserCollection.functions;
    if (!parser || !functionInfo) return;

    if (!(parser instanceof _formulaParser.default)) {
      return;
    }

    var name = functionInfo.name,
        fn = functionInfo.fn;

    if (name && typeof fn === 'function') {
      functions[name] = {
        name: name,
        fn: fn
      };
      parser.setFunction(name, fn);
    }
  };

  this.buildFormVariable = function (variableInfo) {
    var keepPre = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;

    /* 定义主表单变量 */
    _slimLodash.default.forEach(variableInfo, function (variable) {
      /* 定义表单 */
      _this.registerVariable('', variable, keepPre);
    });
  };

  this.registerFormulaSet = function (name) {
    var singleFormula = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    if (!_slimLodash.default.checkEmpty(name)) return;
    var theFormula = _this.formulaSet[name] || {
      referenceFields: [],
      rules: {}
    };
    theFormula.referenceFields = _slimLodash.default.union(theFormula.referenceFields, [singleFormula.referField]);
    theFormula.rules[singleFormula.referField] = singleFormula.rule;
    _this.formulaSet[name] = theFormula;
  };

  this.updateParserPool = function (_ref) {
    var parser = _ref.parser,
        pool = _ref.pool;

    _this.updateParserPoolPrivate({
      parser: parser,
      pool: pool
    });
  };

  this.updateParserPoolPrivate = function (_ref2) {
    var parser = _ref2.parser,
        pool = _ref2.pool;

    if (parser instanceof _formulaParser.default) {
      parser.updateDataPool(pool);
    } else {
      _slimLodash.default.forEach(pool, function (val, key) {
        var matchedVariableInfo = _this.mainParser.variables[key];

        if (val === undefined || val === null || !matchedVariableInfo) {
          return;
        }

        if (typeof matchedVariableInfo.handle === 'function') {
          pool[key] = matchedVariableInfo.handle(val);
        }
      });

      _this.mainParser.parser.updateDataPool(pool);
    }
  };

  this.genFormula = function () {
    var formula = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
    if (!formula) return;
    if (formula.length === 0) return;
    formula.forEach(function (formulaItem) {
      var rule = {
        ruleId: formulaItem.ruleId ? formulaItem.ruleId : "".concat(formulaItem.referField, "-to-").concat(formulaItem.targetField),
        formula: formulaItem.formula,
        disable: formulaItem.disable ? true : false,
        canSetUndefined: formulaItem.canSetUndefined
      };

      if (_slimLodash.default.has(formulaItem, 'updatePool')) {
        rule.updatePool = formulaItem.updatePool;
      }
      /* 是否需要私有编译器 */


      if (formulaItem.privateParser) {
        rule.parserName = formulaItem.parserName || formulaItem.referField;

        _this.registerParser(rule.parserName); //注册私有parser


        _slimLodash.default.forEach(formulaItem.registerVariables, function (varName) {
          _this.registerVariable(rule.parserName, {
            name: varName
          }); //注册私有parser的变量

        });

        if (_slimLodash.default.has(formulaItem, 'setFunctionList')) {
          _slimLodash.default.forEach(formulaItem.setFunctionList, function (fn, key) {
            _this.registerFunction(rule.parserName, {
              name: key,
              fn: fn
            }); //注册私有parser的函数

          });
        }
      } else {
        if (_slimLodash.default.has(formulaItem, 'setFunctionList')) {
          _slimLodash.default.forEach(formulaItem.setFunctionList, function (fn, key) {
            _this.registerFunction(null, {
              name: key,
              fn: fn
            }); //注册私有parser的函数

          });
        }
      }

      _this.registerFormulaSet(formulaItem.targetField, {
        referField: formulaItem.referField,
        rule: rule
      });
    });
  };

  this.triggerFormula = function (referVarName, referVarValue) {
    var formulaResult = {};
    var matchedVariableInfo = _this.mainParser.variables[referVarName];
    if (!matchedVariableInfo) return; //非法的变量不除法改变

    var variableValue = referVarValue;

    if (referVarValue === undefined || referVarValue === null) {
      if (_slimLodash.default.has(matchedVariableInfo, 'defaultValue')) {
        variableValue = matchedVariableInfo.defaultValue;
      }
    } else if (typeof matchedVariableInfo.handle === 'function') {
      variableValue = matchedVariableInfo.handle(referVarValue);
    }

    var effectedVariables = [{
      name: referVarName,
      value: variableValue,
      originalValue: referVarValue || variableValue
    }];
    /* 主编译器原始变量的衍生变量 */

    _slimLodash.default.forEach(_this.mainParser.variables, function (variableInfo, key) {
      var parent = variableInfo.parent,
          handle = variableInfo.handle;

      if (parent === referVarName && typeof handle === 'function') {
        if (!_slimLodash.default.find(effectedVariables, function (o) {
          return o.name === key;
        })) {
          effectedVariables.push({
            name: key,
            value: handle(referVarValue, key)
          });
        }
      }
    });

    _slimLodash.default.forEach(effectedVariables, function (data) {
      /* 更新主编译器 */
      _this.updateParserPoolPrivate({
        parser: _this.mainParser.parser,
        pool: (0, _defineProperty2.default)({}, data.name, data.value)
      });

      _slimLodash.default.forEach(_this.formulaSet, function (formula, key) {
        if (_slimLodash.default.includes(formula.referenceFields, data.name)) {
          var rule = formula.rules[data.name];

          if (rule.parserName) {
            var parserCollect = rule.parserName && _this.childParsers[rule.parserName] ? _this.childParsers[rule.parserName] : _this.mainParser;
            var parser = parserCollect.parser;
            var poolValue;
            /* 私有编译器触发 更新私有updatePool*/

            poolValue = typeof rule.updatePool === 'function' ? rule.updatePool(data.originalValue) : data.originalValue;

            _this.updateParserPoolPrivate({
              parser: parser,
              pool: poolValue
            });
          }
        }
      });
    });

    _slimLodash.default.forEach(effectedVariables, function (data) {
      _slimLodash.default.forEach(_this.formulaSet, function (formula, key) {
        if (_slimLodash.default.includes(formula.referenceFields, data.name)) {
          var rule = formula.rules[data.name];
          if (rule.disable) return;
          var parserCollect = rule.parserName && _this.childParsers[rule.parserName] ? _this.childParsers[rule.parserName] : _this.mainParser;
          var parser = parserCollect.parser;
          var result = parser.parse(rule.formula);

          if (!result.error && (rule.canSetUndefined || result.result !== undefined && result.result !== null)) {
            formulaResult[key] = result.result;
          } else {
            console.warn(result, parserCollect, rule);
          }
        }
      });
    });

    return formulaResult;
  };

  this.runFormula = function (_ref3) {
    var targetField = _ref3.targetField,
        referVarName = _ref3.referVarName,
        referVarValue = _ref3.referVarValue;
    var formulaResult = {};
    var matchedVariableInfo = _this.mainParser.variables[referVarName];
    if (!matchedVariableInfo) return; //非法的变量不除法改变

    var variableValue = referVarValue;

    if (referVarValue === undefined || referVarValue === null) {
      if (_slimLodash.default.has(matchedVariableInfo, 'defaultValue')) {
        variableValue = matchedVariableInfo.defaultValue;
      }
    } else if (typeof matchedVariableInfo.handle === 'function') {
      variableValue = matchedVariableInfo.handle(referVarValue);
    }

    var effectedVariables = [{
      name: referVarName,
      value: variableValue,
      originalValue: referVarValue
    }];
    /* 主编译器原始变量的衍生变量 */

    _slimLodash.default.forEach(_this.mainParser.variables, function (variableInfo, key) {
      var parent = variableInfo.parent,
          handle = variableInfo.handle;

      if (parent === referVarName && typeof handle === 'function') {
        if (!_slimLodash.default.find(effectedVariables, function (o) {
          return o.name === key;
        })) {
          effectedVariables.push({
            name: key,
            value: handle(referVarValue, key)
          });
        }
      }
    });

    _slimLodash.default.forEach(effectedVariables, function (referVariable) {
      /* 更新主编译器 */
      _this.updateParserPoolPrivate({
        parser: _this.mainParser.parser,
        pool: (0, _defineProperty2.default)({}, referVariable.name, referVariable.value)
      });

      var matchedFormula = _this.formulaSet[targetField];

      if (matchedFormula) {
        if (_slimLodash.default.includes(matchedFormula.referenceFields, referVariable.name)) {
          var rule = matchedFormula.rules[referVariable.name];

          if (rule.parserName) {
            var parserCollect = rule.parserName && _this.childParsers[rule.parserName] ? _this.childParsers[rule.parserName] : _this.mainParser;
            var parser = parserCollect.parser;
            var poolValue;
            /* 私有编译器触发 更新私有updatePool*/

            poolValue = typeof rule.updatePool === 'function' ? rule.updatePool(referVariable.originalValue) : referVariable.originalValue;

            _this.updateParserPoolPrivate({
              parser: parser,
              pool: poolValue
            });
          }
        }
      }
    });

    _slimLodash.default.forEach(effectedVariables, function (referVariable) {
      var matchedFormula = _this.formulaSet[targetField];

      if (referVariable.name === referVarName && matchedFormula) {
        if (_slimLodash.default.includes(matchedFormula.referenceFields, referVariable.name)) {
          var rule = matchedFormula.rules[referVariable.name];
          if (rule.disable) return;
          var parserCollect = rule.parserName && _this.childParsers[rule.parserName] ? _this.childParsers[rule.parserName] : _this.mainParser;
          var parser = parserCollect.parser;
          var result = parser.parse(rule.formula);

          if (!result.error && (rule.canSetUndefined || result.result !== undefined && result.result !== null)) {
            formulaResult[targetField] = result.result;
          } else {
            console.warn(result, parserCollect, rule);
          }
        }
      }
    });

    return formulaResult;
  };

  this.buildFormFormula = function (formFields) {
    var preDefinedFormulas = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
    if (formFields && formFields.length === 0) return;
    var formulaRules = preDefinedFormulas || [];
    var shareFields = [];
    /* 目前需要支持共享字段的表单元素 */

    var shareRelatedFields = ['ReceiptCrossInput', 'DeductionInput', 'FormAssociated'];
    /* 数据互联, 外部档案 */

    _slimLodash.default.forEach(formFields, function (item) {
      if (item.type === 'AssociationInput' || item.type === 'ExternalOptionInput') {
        var mapRules = (0, _genAssociatedFormula.default)(item, formFields);
        formulaRules = _slimLodash.default.union(formulaRules, mapRules);
      }
      /* 收集共享字段 */


      if (item.shared) {
        shareFields.push(item);
      }
    });
    /* 共享字段映射分配 */


    if (shareFields.length > 0) {
      _slimLodash.default.forEach(formFields, function (item) {
        if (shareRelatedFields.includes(item.type) && item.property.isMap) {
          var mapRules = (0, _genShareFieldFormula.default)(item, shareFields);
          formulaRules = _slimLodash.default.union(formulaRules, mapRules);
        }
      });
    }
    /* 公式字段映射分配 */


    var formulaFieldRules = (0, _genFormulaFieldFormula.default)(formFields);

    var _ref4 = formulaFieldRules || {},
        variableList = _ref4.variableList,
        formula = _ref4.formula;

    if (formulaFieldRules) {
      if (formula && formula.length > 0) {
        formulaRules = _slimLodash.default.union(formulaRules, formula);
      }
    }

    _this.buildFormVariable(variableList);

    _this.genFormula(formulaRules);
  };

  this.updateFormFormula = function (formFields) {
    var preDefinedFormulas = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];

    _this.buildFormFormula(formFields, preDefinedFormulas = [], true);
  };

  this.updatePoolInFormula = function () {
    var data = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    var parserName = arguments.length > 1 ? arguments[1] : undefined;

    _slimLodash.default.forEach(_this.formulaSet, function (formula, key) {
      var rules = formula.rules;

      _slimLodash.default.forEach(rules, function (rule, referName) {
        if (rule.parserName) {
          var shouldUpdate = false;
          /*REVIEW 如果parserName没提供，则更新所有私有编译器的数据池,是否有这个必要？ */

          if (parserName) {
            if (rule.parserName === parserName) shouldUpdate = true;
          } else {
            shouldUpdate = true;
          }

          if (shouldUpdate) {
            var parserCollect = rule.parserName && _this.childParsers[rule.parserName];

            if (referName in data && parserCollect) {
              var poolValue = typeof rule.updatePool === 'function' ? rule.updatePool(data[referName]) : (0, _defineProperty2.default)({}, "".concat(referName), data[referName]);

              _this.updateParserPoolPrivate({
                parser: parserCollect.parser,
                pool: poolValue
              });
            }
          }
        }
      });
    });
  };

  this.setFormulaUseable = function (ruleData, parserName) {
    if (_slimLodash.default.isEmpty(ruleData)) return;

    _slimLodash.default.forEach(_this.formulaSet, function (formula) {
      var rules = formula.rules;

      _slimLodash.default.forEach(rules, function (rule) {
        if ('ruleId' in rule === false) return;

        var matchedRule = _slimLodash.default.find(ruleData, function (v) {
          return v.ruleId === rule.ruleId;
        });

        if (matchedRule) {
          if (parserName) {
            if (rule.parserName === parserName) {
              rule.disable = !matchedRule.useable;
            }
          } else {
            rule.disable = !matchedRule.useable;
          }
        }
      });
    });
  };

  this.mainParser = {
    name: 'mainForm',
    parser: new _formulaParser.default(),
    variables: {},
    functions: {}
  };
  this.childParsers = {};
  this.formulaSet = {};
}
/* 注册变量 */
;

var _default = FormChefParser;
exports.default = _default;