"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _slimLodash = _interopRequireDefault(require("../utils/slimLodash"));

/*
 * @Description: file content
 * @Author: RongWei
 * @Date: 2019-08-20 15:17:46
 * @LastEditors: RongWei
 * @LastEditTime: 2019-09-18 19:50:11
 */

/**
 * 用于生成数据互联的规则
 */
function _getManualOption(value) {
  var manualItems = this.field && this.field.property.manualItems;
  if (manualItems && manualItems.includes(value)) return value;
}
/* 数据互联字段映射 */


function genAssociatedFormula(field, formFields) {
  var result = [];

  if (field && field.property && !field.property.isMulti) {
    _slimLodash.default.forEach(field.associationDataRelations, function (item) {
      var targetField = _slimLodash.default.find(formFields, function (o) {
        return o.identifier === item.identifier;
      });

      var formula = '';

      if (targetField && targetField.type === 'DateTimeInput' && targetField.property.selectionModule === 'SINGLE') {
        formula = "mkDate(".concat(item.businessCode, ")");
      } else if (targetField && targetField.type === 'OptionInput') {
        if (['MANUAL'].includes(targetField.property.sourceMode)) formula = "getManualOption(".concat(item.businessCode, ")");
      } else {
        formula = item.businessCode;
      }

      result.push({
        targetField: item.identifier,
        referField: field.identifier,
        formula: formula,
        setFunctionList: {
          mkDate: function mkDate(value) {
            // 数据互联无默认值时，value值不合法，需要转换判断一下
            return value && !Number.isNaN(new Date(value).valueOf()) ? {
              currentTime: value
            } : {
              currentTime: undefined
            };
          },
          getManualOption: function getManualOption(value) {
            return _getManualOption.call({
              field: targetField
            }, value);
          }
        },
        privateParser: true,
        canSetUndefined: true,
        updatePool: function updatePool(fieldValue) {
          // fieldValue值为undefined时，无法覆盖联动的值。
          var value = fieldValue && fieldValue.value && fieldValue.value[0];

          if (value) {
            return value;
          }

          value = {};

          if (fieldValue && fieldValue.definedList) {
            _slimLodash.default.forEach(fieldValue.definedList, function (item) {
              value[item.businessCode] = null;
            });
          }

          return value || null;
        }
      });
    });
  }

  return result;
}

var _default = genAssociatedFormula;
exports.default = _default;