"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _lodash = _interopRequireDefault(require("lodash"));

/* 发票和费用金额的公式关系 */
function genFormula(options) {
  var sourceType = options.sourceType,
      isCurrentCorp = options.isCurrentCorp,
      _options$formFields = options.formFields,
      formFields = _options$formFields === void 0 ? [] : _options$formFields;
  var formulaRules = [];
  var hasInvoideField = _lodash.default.findIndex(formFields, {
    identifier: 'invoiceList'
  }) !== -1; // 第三方和企业导入订单不联动发票金额且

  if (sourceType || !hasInvoideField) return [];

  if (isCurrentCorp) {
    /* 对公费用 */
    formulaRules.push({
      targetField: 'receiptAmount',
      referField: 'invoiceList',
      formula: 'invoiceList',
      registerVariables: ['invoiceList'],
      privateParser: true,
      updatePool: function updatePool(fieldValue) {
        // 计算发票的总金额
        if (!fieldValue || !fieldValue.list) return {
          invoiceList: undefined
        };

        if (fieldValue._isSetFieldsValue) {
          delete fieldValue.list._isSetFieldsValue;
          return {
            invoiceList: undefined
          };
        }

        var amount = fieldValue.list.reduce(function (total, _ref) {
          var totalPriceAndTax = _ref.totalPriceAndTax;
          return total + (totalPriceAndTax || 0);
        }, 0); // return { invoiceList: { currency: 'CNY', amount: Math.round(amount * 1000) / 1000 } };

        return {
          invoiceList: Math.round(amount * 1000) / 1000
        };
      }
    });
  } else {
    formulaRules.push({
      targetField: 'consumeAmount',
      referField: 'invoiceList',
      formula: 'invoiceList',
      registerVariables: ['invoiceList'],
      privateParser: true,
      updatePool: function updatePool(fieldValue) {
        // 计算发票的总金额
        if (!fieldValue || !fieldValue.list) return {
          invoiceList: undefined
        };

        if (fieldValue._isSetFieldsValue) {
          delete fieldValue.list._isSetFieldsValue;
          return {
            invoiceList: undefined
          };
        }

        var amount = fieldValue.list.reduce(function (total, _ref2) {
          var totalPriceAndTax = _ref2.totalPriceAndTax;
          return total + (totalPriceAndTax || 0);
        }, 0); // return { invoiceList: { currency: 'CNY', amount: Math.round(amount * 1000) / 1000 } };

        return {
          invoiceList: Math.round(amount * 1000) / 1000
        };
      }
    });
  }

  return formulaRules;
}

var _default = genFormula;
exports.default = _default;