"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = bindEventListeners;

var _globals = require("./globals");

var _hideAllPoppers = _interopRequireDefault(require("./hideAllPoppers"));

var _closest = _interopRequireDefault(require("../utils/closest"));

var _find = _interopRequireDefault(require("../utils/find"));

var _matches = require("../utils/matches");

var _utils = _interopRequireDefault(require("../../../utils/utils"));

var prefix = _utils.default.prefixCls;
/**
* Adds the needed event listeners
*/

function bindEventListeners() {
  var touchHandler = function touchHandler() {
    _globals.Browser.touch = true;

    if (_globals.Browser.iOS()) {
      document.body.classList.add("".concat(prefix, "-tippy-touch"));
    }

    if (_globals.Browser.dynamicInputDetection && window.performance) {
      document.addEventListener('mousemove', mousemoveHandler);
    }
  };

  var mousemoveHandler = function () {
    var time;
    return function () {
      var now = performance.now(); // Chrome 60+ is 1 mousemove per rAF, use 20ms time difference

      if (now - time < 20) {
        _globals.Browser.touch = false;
        document.removeEventListener('mousemove', mousemoveHandler);

        if (!_globals.Browser.iOS()) {
          document.body.classList.remove("".concat(prefix, "-tippy-touch"));
        }
      }

      time = now;
    };
  }();

  var clickHandler = function clickHandler(event) {
    // Simulated events dispatched on the document
    if (!(event.target instanceof Element)) {
      return (0, _hideAllPoppers.default)();
    }

    var el = (0, _closest.default)(event.target, _globals.Selectors.TOOLTIPPED_EL);
    var popper = (0, _closest.default)(event.target, _globals.Selectors.POPPER);

    if (popper) {
      var ref = (0, _find.default)(_globals.Store, function (ref) {
        return ref.popper === popper;
      });
      if (!ref) return;
      var interactive = ref.settings.interactive;
      if (interactive) return;
    }

    if (el) {
      var _ref = (0, _find.default)(_globals.Store, function (ref) {
        return ref.el === el;
      });

      if (!_ref) return;
      var _ref$settings = _ref.settings,
          hideOnClick = _ref$settings.hideOnClick,
          multiple = _ref$settings.multiple,
          trigger = _ref$settings.trigger; // Hide all poppers except the one belonging to the element that was clicked IF
      // `multiple` is false AND they are a touch user, OR
      // `multiple` is false AND it's triggered by a click

      if (!multiple && _globals.Browser.touch || !multiple && trigger.indexOf('click') !== -1) {
        return (0, _hideAllPoppers.default)(_ref);
      } // If hideOnClick is not strictly true or triggered by a click don't hide poppers


      if (hideOnClick !== true || trigger.indexOf('click') !== -1) return;
    } // Don't trigger a hide for tippy controllers, and don't needlessly run loop


    if ((0, _closest.default)(event.target, _globals.Selectors.CONTROLLER) || !document.querySelector(_globals.Selectors.POPPER)) return;
    (0, _hideAllPoppers.default)();
  };

  var blurHandler = function blurHandler(event) {
    var _document = document,
        el = _document.activeElement;

    if (el && el.blur && _matches.matches.call(el, _globals.Selectors.TOOLTIPPED_EL)) {
      el.blur();
    }
  }; // Hook events


  document.addEventListener('click', clickHandler);
  document.addEventListener('touchstart', touchHandler);
  window.addEventListener('blur', blurHandler);

  if (!_globals.Browser.SUPPORTS_TOUCH && (navigator.maxTouchPoints > 0 || navigator.msMaxTouchPoints > 0)) {
    document.addEventListener('pointerdown', touchHandler);
  }
}