"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = BodyTable;

var _extends2 = _interopRequireDefault(require("babel-runtime/helpers/extends"));

var _react = _interopRequireDefault(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _utils = require("./utils");

var _reactPerfectScrollbar = _interopRequireDefault(require("react-perfect-scrollbar"));

var _BaseTable = _interopRequireDefault(require("./BaseTable"));

function BodyTable(props, _ref) {
  var table = _ref.table;
  var _table$props = table.props,
      prefixCls = _table$props.prefixCls,
      scroll = _table$props.scroll;
  var columns = props.columns,
      fixed = props.fixed,
      tableClassName = props.tableClassName,
      getRowKey = props.getRowKey,
      handleBodyScroll = props.handleBodyScroll,
      localScrollTable = props.localScrollTable,
      expander = props.expander,
      isAnyColumnsFixed = props.isAnyColumnsFixed;
  var saveRef = table.saveRef;
  var useFixedHeader = table.props.useFixedHeader;
  var bodyStyle = (0, _extends2.default)({}, table.props.bodyStyle);
  var innerBodyStyle = {};

  if (scroll.x || fixed) {
    bodyStyle.overflowX = bodyStyle.overflowX || 'scroll'; // Fix weired webkit render bug
    // https://github.com/ant-design/ant-design/issues/7783

    bodyStyle.WebkitTransform = 'translate3d (0, 0, 0)';
  }

  if (scroll.y) {
    // maxHeight will make fixed-Table scrolling not working
    // so we only set maxHeight to body-Table here
    if (fixed) {
      innerBodyStyle.maxHeight = bodyStyle.maxHeight || scroll.y;
      innerBodyStyle.overflowY = bodyStyle.overflowY || 'scroll';
    } else {
      bodyStyle.maxHeight = bodyStyle.maxHeight || scroll.y;
    }

    bodyStyle.overflowY = bodyStyle.overflowY || 'scroll';
    useFixedHeader = true; // Add negative margin bottom for scroll bar overflow bug

    var scrollbarWidth = (0, _utils.measureScrollbar)();

    if (scrollbarWidth > 0 && fixed) {
      bodyStyle.marginBottom = '-' + scrollbarWidth + 'px';
      bodyStyle.paddingBottom = '0px';
    }
  }

  var baseTable = /*#__PURE__*/_react.default.createElement(_BaseTable.default, {
    tableClassName: tableClassName,
    hasHead: !useFixedHeader,
    hasBody: true,
    fixed: fixed,
    columns: columns,
    expander: expander,
    getRowKey: getRowKey,
    isAnyColumnsFixed: isAnyColumnsFixed
  });

  if (fixed && columns.length) {
    var refName = void 0;

    if (columns[0].fixed === 'left' || columns[0].fixed === true) {
      refName = 'fixedColumnsBodyLeft';
    } else if (columns[0].fixed === 'right') {
      refName = 'fixedColumnsBodyRight';
    }

    delete bodyStyle.overflowX;
    delete bodyStyle.overflowY;
    return /*#__PURE__*/_react.default.createElement('div', {
      key: 'bodyTable',
      className: prefixCls + '-body-outer',
      onMouseEnter: function onMouseEnter() {
        localScrollTable(refName);
      },
      onMouseLeave: function onMouseLeave() {
        localScrollTable('');
      }
    }, /*#__PURE__*/_react.default.createElement(_reactPerfectScrollbar.default, {
      className: "".concat(prefixCls, "-body-inner freeze-table"),
      containerRef: function ref(_ref2) {
        saveRef(refName)(_ref2);
      },
      onScrollY: function onScrollY(e) {
        handleBodyScroll(e, refName);
      },
      onScrollX: function onScrollX(e) {
        handleBodyScroll(e, refName);
      }
    }, baseTable));
  } else {
    return /*#__PURE__*/_react.default.createElement(_reactPerfectScrollbar.default, {
      key: 'bodyTable',
      className: "".concat(prefixCls, "-body main-table"),
      containerRef: function ref(_ref3) {
        saveRef('bodyTable')(_ref3);
      },
      onScrollX: function onScrollX(e) {
        handleBodyScroll(e, 'bodyTable');
      },
      onScrollY: function onScrollY(e) {
        handleBodyScroll(e, 'bodyTable');
      }
    }, baseTable); // return React.createElement(
    //   PerfectScrollbar,
    //   {
    //     key: 'bodyTable',
    //     className: prefixCls + '-body'
    //     , ref: function ref(_ref3) {
    //       saveRef('bodyTable')(_ref3 && _ref3._container);
    //     },
    //     onScrollX: (e) => { handleBodyScroll(e, 'bodyTable') },
    //     onScrollY: (e) => { handleBodyScroll(e, 'bodyTable') },
    //   },
    //   baseTable
    // );
  }
}

BodyTable.propTypes = {
  fixed: _propTypes.default.oneOfType([_propTypes.default.string, _propTypes.default.bool]),
  columns: _propTypes.default.array.isRequired,
  tableClassName: _propTypes.default.string.isRequired,
  handleWheel: _propTypes.default.func.isRequired,
  handleBodyScroll: _propTypes.default.func.isRequired,
  getRowKey: _propTypes.default.func.isRequired,
  expander: _propTypes.default.object.isRequired,
  isAnyColumnsFixed: _propTypes.default.bool
};
BodyTable.contextTypes = {
  table: _propTypes.default.any
};