"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _lodash = _interopRequireDefault(require("lodash"));

/*
 * @Description: file content
 * @Author: RongWei
 * @Date: 2020-07-13 19:09:41
 * @LastEditors: RongWei
 * @LastEditTime: 2020-07-13 19:12:24
 */

/* 有向无环图Directed Acyclic Graph简称DAG,拓扑次序(Topological Order) */
var DependenceFieldsSort = function DependenceFieldsSort() {
  var _this = this;

  (0, _classCallCheck2.default)(this, DependenceFieldsSort);

  this.buildGraph = function (associationFields) {
    var targetFiels = _lodash.default.map(associationFields, 'identifier');

    _lodash.default.each(associationFields, function (field) {
      var referFields = field.referFields;
      var referFieldsLength = referFields.length || 0;
      var _cascadeFields = [];

      for (var i = 0; i < referFieldsLength; i++) {
        // 若数据互联依赖了数据互联之外的其他组件，则不考虑在内
        if (targetFiels.includes(referFields[i])) {
          _cascadeFields.push(referFields[i]);
        }
      }

      field.calcReferFields = (0, _toConsumableArray2.default)(referFields);

      if (_cascadeFields.length) {
        _this.surplusArr.push(field);
      } else {
        _this.noDependArr.push(field);
      }
    });

    _this.sortCascade();
  };

  this.sortCascade = function () {
    var _loop = function _loop() {
      var targetFiels = _lodash.default.map(_this.noDependArr, 'identifier');

      if (_this.noDependArr.length) {
        //移除对外无关属性calcReferFields
        _lodash.default.forEach(_this.noDependArr, function (v) {
          delete v.calcReferFields;
        }); // 将无依赖的数组放入 sortedList 中


        _this.sortedList.push(_this.noDependArr);

        _this.noDependArr = [];
      } // 对于入度不为0的顶点，将n从他们的依赖中删除，如果删除后入度为0，则将其添加到noDependArr中


      for (var i = 0; i < _this.surplusArr.length; i++) {
        var calcReferFields = _this.surplusArr[i].calcReferFields; // 在 calcReferFields 中删除依赖了 noDependArr 的 item

        _lodash.default.remove(calcReferFields, function (item) {
          return targetFiels.includes(item);
        }); // 删除后若无依赖，则放入 noDependArr 中


        if (!calcReferFields.length) {
          _this.noDependArr.push(_this.surplusArr[i]);

          _this.surplusArr.splice(i, 1);

          i--;
        }
      }
    };

    while (_this.noDependArr.length) {
      _loop();
    }
  };

  this.noDependArr = [];
  this.surplusArr = [];
  this.sortedList = [];
} // 构建数据
;

var _default = DependenceFieldsSort;
exports.default = _default;